/*
 * Linguagem: Linguagem para Definicao de Ontologias
 * Processador: Gramatica de atributos de Contexto que permite definir \s
                Valida: Declaracoes Repetidas de conceitos, individuos, relacoes e triplos
                Valida: Elementos de cada Triplo (geral e casos particulares)
                Valida: Instanciacao de atributos nos Triplos com Heranca de Atributos nas SubClasses
                Gera: se nao houver erros semanticos, cria uma representacao grafica  em DOT da ontologia
                Gera: se nao houver erros semanticos, cria uma representacao em OWL da ontologia
                Grava os outputs gerados em ficheiros diferentes
 * PRH+CEA 2018.01.03
 */

grammar OntoDLProcessor;

@header{
        import java.util.*;
        import java.nio.file.*;
        import java.nio.file.StandardOpenOption;
        import java.nio.file.attribute.FileAttribute;
        import java.nio.file.attribute.PosixFilePermission;
        import java.nio.file.attribute.PosixFilePermissions;
}

ontologia  returns[String name, HashSet<String> concts , HashSet<String> indivs ,  HashSet<String> rels,  HashSet<String> tripls,
                   HashMap<String,List<String>> tabConcs,
                   boolean erroSem, boolean erroConcts, boolean erroIndivs, boolean erroRels, boolean erroTriplos]
@init{ String outputDOTindivs; String outputOWLindivs;}
             : 'Ontologia' TEXT {
                                 $ontologia.name=$TEXT.text;
                                }
                    conceitos   { $ontologia.concts=$conceitos.concts;
                                  $ontologia.erroConcts=$conceitos.erroConcts;
                                  System.out.println("Step1-declaracao de conceitos: "+$conceitos.tabConcs); //debug only
                                }
                    individuos? { if ($individuos.text==null) {
                                      outputDOTindivs = "";
                                      outputOWLindivs = "";
                                      $ontologia.indivs=new HashSet<>();
                                      $ontologia.erroIndivs=false; }
                                  else {
                                      outputDOTindivs = $individuos.outputDOT;
                                      outputOWLindivs = $individuos.outputOWL;
                                      $ontologia.indivs=$individuos.indivs;
                                      $ontologia.erroIndivs=$individuos.erroIndivs;}
                                  }
                    relacoes    { $ontologia.rels=$relacoes.rels;
                                  $ontologia.erroRels=$relacoes.erroRels; }
                    triplos[$ontologia.concts,$ontologia.indivs,$ontologia.rels,$conceitos.tabConcs]
                                { $ontologia.tripls   = $triplos.tripls;
                                  $ontologia.tabConcs = $triplos.tabConcsout;
                                  System.out.println("Step2-heranca de subclasses: "+$ontologia.tabConcs); //debug only

                                  $ontologia.erroTriplos=$triplos.erroTriplos;
                                  $ontologia.erroSem=($ontologia.erroConcts || $ontologia.erroIndivs || $ontologia.erroRels || $ontologia.erroTriplos);

                                  if (!$ontologia.erroSem){/*"/Users/PRH/Dropbox/PRH/MScMarLeal"*/
                                      String workDir = "/Users/Cristiana Ara�jo/Desktop";
                                      System.out.println("----------------DOT------------------");
                                      String cabecalhoDOT = "digraph  " +$ontologia.name+ "{\n";
                                      //System.out.println("digraph  " +$ontologia.name+ "{");
                                      //System.out.println(outputDOTindivs);
                                      //System.out.println($conceitos.outputDOT);
                                      //System.out.println($triplos.outputDOT);
                                      String rodapeDOT = "}";
                                      String DOT = cabecalhoDOT + outputDOTindivs + $conceitos.outputDOT + $triplos.outputDOT + rodapeDOT;
                                      //System.out.println("}");
                                      //Path cc = Paths.get("");
                                      //System.out.println("CC: "+cc.toAbsolutePath().toString());
                                      System.out.println(DOT);
                                      //Path DOTFileName = Paths.get("/Users/giulianimartini/Downloads/"+$ontologia.name+".dot");
                                      Path DOTFileName = Paths.get(workDir+"/"+$ontologia.name+".dot");
                                      Set<PosixFilePermission> perms = PosixFilePermissions.fromString("rwxrwxrwx");
                                      FileAttribute<Set<PosixFilePermission>> attr = PosixFilePermissions.asFileAttribute(perms);
                                      try {
                                        if (!Files.exists(DOTFileName)) {
                                            Files.createFile(DOTFileName);
                                        } else {
                                            Files.delete(DOTFileName);
                                        }
                                        Files.write(DOTFileName, DOT.getBytes(), StandardOpenOption.CREATE);
                                        //Files.setPosixFilePermissions(DOTFileName, perms);
                                      } catch(Exception e) { e.printStackTrace(); }
                                      /*-------------------------------------------------------*/
                                      System.out.println("----------------OWL------------------");
                                      String cabecalho = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n" +
                                      "<!DOCTYPE Ontology [\n" +
                                      "\t<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">\n" +
                                      "\t<!ENTITY rdf \"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n" +
                                      "\t<!ENTITY rdfs \"http://www.w3.org/2000/01/rdf-schema#\">\n" +
                                      "\t<!ENTITY owl \"http://www.w3.org/2002/07/owl#\">\n" +
                                      "]>\n\n" + "<Ontology>";
                                      System.out.println(cabecalho);
                                      System.out.println(outputOWLindivs);
                                      System.out.println($conceitos.outputOWL);
                                      System.out.println($relacoes.outputOWL);
                                      System.out.println($triplos.outputOWL);
                                      String rodape = "</Ontology>";
                                      System.out.println(rodape);
                                      String owlFinalFile = cabecalho + outputOWLindivs + $conceitos.outputOWL + $relacoes.outputOWL + $triplos.outputOWL + rodape;
                                      //Path owlFileName = Paths.get("/Users/giulianimartini/Downloads/"+$ontologia.name+".owl");
                                      Path owlFileName = Paths.get(workDir+"/"+$ontologia.name+".owl");
                                      //Set<PosixFilePermission> perms = PosixFilePermissions.fromString("rwxrwxrwx");
                                      //FileAttribute<Set<PosixFilePermission>> attr = PosixFilePermissions.asFileAttribute(perms);
                                      try {
                                        if (!Files.exists(owlFileName)) {
                                            Files.createFile(owlFileName);
                                        } else {
                                            Files.delete(owlFileName);
                                        }
                                        Files.write(owlFileName, owlFinalFile.getBytes(), StandardOpenOption.CREATE);
                                       //Files.setPosixFilePermissions(owlFileName, perms);
                                      } catch(Exception e) { e.printStackTrace(); }
                                  }
                                  else System.out.println("Foram encontrados (1 ou mais) Erros Semanticos! descarte qualquer resultado e corrija a sua espeficacao!"); }
                 '.'
           ;

conceitos  returns[HashSet<String> concts, boolean erroConcts, String outputDOT, String outputOWL, HashMap<String,List<String>> tabConcs]
           : 'conceitos'
             '{' c1=conceito[new HashMap<String,List<String>>()]
                              { $conceitos.concts = new HashSet<>();
                                $conceitos.concts.add($c1.nome);
                                $conceitos.erroConcts = false;
                                $conceitos.tabConcs = $c1.tabConcsout;
                                $conceitos.outputDOT = $c1.outputDOT;
                                $conceitos.outputOWL = $c1.outputOWL;}
             (',' c2=conceito[$conceitos.tabConcs]
                              {
                                if($conceitos.concts.contains($c2.nome)){
                                  $conceitos.erroConcts = true;
                                  System.out.println("ERRO: Ja existe um conceito com o nome <"+ $c2.nome + ">");}
                                else {
                                  $conceitos.concts.add($c2.nome);
                                  $conceitos.tabConcs = $c2.tabConcsout;
                                  $conceitos.outputDOT = $conceitos.outputDOT +$c2.outputDOT;
                                  $conceitos.outputOWL = $conceitos.outputOWL + $c2.outputOWL;}
                                } )*
             '}'
           ;


conceito   [HashMap<String,List<String>> tabConcsin]
           returns[String nome, String outputDOT, String outputOWL, HashMap<String,List<String>> tabConcsout]
           : TEXT { $conceito.nome=$TEXT.text;
                    $conceito.outputDOT = "\"" +$conceito.nome+ "\" [shape = ellipse, style=filled, color=turquoise4]; \n";
                    $conceito.outputOWL = "\n\t<Declaration>\n\t\t<Class IRI=\"#"+ $conceito.nome +"\"/>\n\t</Declaration>\n";}
             atributos[$conceito.nome]?
             { if (!($atributos.text==null)){
                           $conceito.tabConcsin.put($conceito.nome,$atributos.lstAtribs);
                           $conceito.tabConcsout = $conceito.tabConcsin;

                           $conceito.outputDOT = "\"" +$conceito.nome+ "\" [shape = ellipse, style=filled, color=turquoise4]; \n";
                           $conceito.outputDOT =  $conceito.outputDOT +  $atributos.outputDOT;
                           $conceito.outputDOT = $conceito.outputDOT + "\t\"" + $conceito.nome + "\"->\"" + $atributos.atribs + "\" [label=\"Properties\" style=dotted color=red]\n";
                           $conceito.outputOWL = $conceito.outputOWL + $atributos.outputOWL;
                    }
               else { $conceito.tabConcsout = $conceito.tabConcsin;
                      $conceito.outputDOT = "\"" +$conceito.nome+ "\" [shape = ellipse, style=filled, color=turquoise4]; \n";

                    }
            }
           ;

atributos  [String concNome]
          returns [String atribs, List<String> lstAtribs, String outputDOT, String outputOWL]
@init { $atributos.lstAtribs = new ArrayList<String>(); }
           : '[' a1=atributo[$atributos.lstAtribs, $atributos.concNome]
                             { $atributos.atribs = $a1.atrib;
                               $atributos.lstAtribs = $a1.lstAtribsout;
                               $atributos.outputOWL = $a1.outputOWL;}
             (','a2=atributo[$atributos.lstAtribs, $atributos.concNome]
                             { $atributos.atribs = $atributos.atribs + " ; "  + $a2.atrib;
                               $atributos.lstAtribs = $a2.lstAtribsout;
                               $atributos.outputOWL = $atributos.outputOWL +  $a2.outputOWL;} )*
             ']' {$atributos.outputDOT =  "\"" +$atributos.atribs+ "\" [shape=rectangle,color=turquoise4]; \n";}
           ;

atributo   [List<String> lstAtribsin, String concNome]
           returns [String atrib, List<String> lstAtribsout, String outputOWL]
           : atr=TEXT ':' tipo=TEXT
             { $atributo.atrib = $atr.text + " : " + $tipo.text;
               $atributo.lstAtribsin.add($atr.text);
               $atributo.lstAtribsout = $atributo.lstAtribsin;
               $atributo.outputOWL = "\n\t<Declaration>\n\t\t<DataProperty IRI=\"#"+ $atr.text + "\"/>\n\t</Declaration>\n";
               $atributo.outputOWL = $atributo.outputOWL + "\n\t<DataPropertyDomain>\n\t\t<DataProperty IRI=\"#"+ $atr.text + "\"/>\n\t\t<Class IRI=\"#"+ $atributo.concNome + "\"/>\n\t</DataPropertyDomain>\n"
                                               + "\t<DataPropertyRange>\n\t\t<DataProperty IRI=\"#"+  $atr.text + "\"/>\n\t\t<Datatype abbreviatedIRI=\"xsd:"+  $tipo.text + "\"/>\n\t</DataPropertyRange>\n";
             }
           ;

individuos returns[HashSet<String> indivs, boolean erroIndivs, String outputDOT, String outputOWL ]
           : 'individuos'
             '{' TEXT   TEXT?{ $individuos.indivs=new HashSet<>();
                         $individuos.indivs.add($TEXT.text);
                         $individuos.erroIndivs = false;
                         $individuos.outputDOT = "\"" +$TEXT.text+ "\" [shape = rectangle, style=filled, color=goldenrod]; \n";
                         $individuos.outputOWL = "\n\t<Declaration>\n\t\t<NamedIndividual IRI=\"#"+ $TEXT.text + "\"/>\n\t</Declaration>\n"; }
             (',' TEXT TEXT?{
                         if($individuos.indivs.contains($TEXT.text)){
                            $individuos.erroIndivs = true;
                            System.out.println("ERRO: Ja existe um individuo com o nome <"+ $TEXT.text + ">"); }
                         else {
                            $individuos.indivs.add($TEXT.text);
                            $individuos.outputDOT = $individuos.outputDOT + "\"" +$TEXT.text+ "\" [shape = rectangle, style=filled, color=goldenrod]; \n"; }
                            $individuos.outputOWL = $individuos.outputOWL + "\n\t<Declaration>\n\t\t<NamedIndividual IRI=\"#"+ $TEXT.text + "\"/>\n\t</Declaration>\n";
                         } )*
             '}'
           ;

relacoes   returns [HashSet<String> rels, boolean erroRels, String outputOWL ]
    @init{$relacoes.rels=new HashSet<>();
          $relacoes.rels.add("isa");
          $relacoes.rels.add("iof");
          $relacoes.rels.add("pof");
          $relacoes.rels.add("syn");
          $relacoes.outputOWL = "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#is-a\"/>\n\t</Declaration>\n" +
                                "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#iof\"/>\n\t</Declaration>\n" +
                                "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#pof\"/>\n\t</Declaration>\n" +
                                "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#syn\"/>\n\t</Declaration>\n";
         }
           : 'relacoes'
             '{' TEXT    { $relacoes.rels.add($TEXT.text);
                           $relacoes.erroRels = false;
                           $relacoes.outputOWL = $relacoes.outputOWL + "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#"+ $TEXT.text + "\"/>\n\t</Declaration>\n";}
             (',' TEXT   {
                           if($relacoes.rels.contains($TEXT.text)){
                             $relacoes.erroRels = true;
                             System.out.println("ERRO: Ja existe uma relacao com o nome <"+ $TEXT.text + ">");}
                           else $relacoes.rels.add($TEXT.text);
                                $relacoes.outputOWL = $relacoes.outputOWL  + "\n\t<Declaration>\n\t\t<ObjectProperty IRI=\"#"+ $TEXT.text + "\"/>\n\t</Declaration>\n";
                           } )*
             '}'
           ;

triplos   [HashSet<String> concts, HashSet<String> indivs, HashSet<String> rels, HashMap<String,List<String>> tabConcsin]
          returns[HashSet<String> tripls, boolean erroTriplos, HashMap<String,List<String>> tabConcsout,
                  String outputDOT, String outputOWL]
          : 'triplos'
             '{' triplo[$triplos.concts,$triplos.indivs,$triplos.rels,$triplos.tabConcsin,"",""]  // ('('triplo[$triplos.concts,$triplos.indivs,$triplos.rels,$triplos.tabConcsin,"",""]')') | triplo[$triplos.concts,$triplos.indivs,$triplos.rels,$triplos.tabConcsin,"",""]
                         { $triplos.tripls=new HashSet<>();
                           $triplos.tripls.add($triplo.trip);
                           $triplos.erroTriplos = $triplo.erroTriplo;
                           $triplos.tabConcsout = $triplo.tabConcsout;
                           $triplos.outputDOT = $triplo.outputDOTout;
                           $triplos.outputOWL = $triplo.outputOWLout;}
             (';' triplo[$triplos.concts,$triplos.indivs,$triplos.rels,$triplos.tabConcsout,$triplos.outputDOT,$triplos.outputOWL]
                         {
                           if($triplos.tripls.contains($triplo.trip))
                             { $triplos.erroTriplos = ($triplos.erroTriplos || true);
                               System.out.println("ERRO: Triplo repetido <"+$triplo.trip+">");}
                           else $triplos.tripls.add($triplo.trip);
                           $triplos.erroTriplos = ($triplos.erroTriplos || $triplo.erroTriplo);
                           $triplos.tabConcsout = $triplo.tabConcsout;
                           $triplos.outputDOT =  $triplo.outputDOTout;
                           $triplos.outputOWL =  $triplo.outputOWLout;}
             )* '}'
           ;

triplo     [HashSet<String> concts, HashSet<String> indivs, HashSet<String> rels, HashMap<String,List<String>> tabConcsin, String outputDOTin, String outputOWLin]
           returns [String trip, boolean erroTriplo, HashMap<String,List<String>> tabConcsout, String outputDOTout, String outputOWLout]
@init { List<String> lstAtribs = new ArrayList<String>();
        int numAtribsPrev=0; int numAtribsReal=0;
        Boolean erroAtribs = false;}
           : '('? c1=TEXT '=' ('iof'|'instancia') '=>' c2=TEXT atribInsts[$c2.text]? ')'?
                  { if (!($triplo.indivs.contains($c1.text)))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> de IOF Nao E um Individuo"); }
                    if (!($triplo.concts.contains($c2.text)))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O objeto <"+$c2.text+"> de IOF Nao E um Conceito"); }
                    $triplo.trip = $c1.text + ",iof," + $c2.text;
                    if (!$triplo.erroTriplo){
                       if ($triplo.tabConcsin.containsKey($c2.text))
                          { lstAtribs = $triplo.tabConcsin.get($c2.text);
                            numAtribsPrev = lstAtribs.size(); }
                       //System.out.println("Este conceito "+$c2.text+" tem "+numAtribsPrev+" atributos: "+lstAtribs); //debug only
                       if (!($atribInsts.text==null)) { numAtribsReal = $atribInsts.conta; }
                       erroAtribs=(numAtribsPrev!=numAtribsReal);
                       if (erroAtribs) { System.out.println("ERRO: O numero de Atributos ("+numAtribsReal+") da Instancia <"+$c1.text+"> NAO concorda com o numero de Atributos ("+numAtribsPrev+") do respetivo Conceito <"+$c2.text+">"); }
                       if ((!erroAtribs)&&(numAtribsReal>0)) {
                          for(String a : $atribInsts.atrs) { if (lstAtribs.indexOf(a)==-1) { erroAtribs=erroAtribs||true;
                                                                 System.out.println("ERRO: O Atributo <"+a+"> associado a Instancia <"+$c1.text+"> NAO consta da lista de Atributos do respetivo Conceito <"+$c2.text+">"); }}
                          }
                       $triplo.erroTriplo=(($triplo.erroTriplo)||(erroAtribs));
                    }
                    if (!$triplo.erroTriplo){
                       $triplo.outputDOTout = $triplo.outputDOTin + "\t\"" + $c1.text + "\"->\"" + $c2.text + "\" [label=\"iof\" style=dashed]\n";
                       $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ClassAssertion>\n\t\t<Class IRI=\"#"+ $c2.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c1.text + "\"/>\n\t</ClassAssertion>\n";
                       if (!($atribInsts.text==null)){
                           $triplo.outputDOTout = $triplo.outputDOTout +  $atribInsts.outputDOT ;
                           $triplo.outputDOTout = $triplo.outputDOTout + "\t\"" + $c1.text + "\"->\"" + $atribInsts.atribsI + "\" [label=\"properties\" style=dotted color=red]\n";
                           $triplo.outputOWLout = $triplo.outputOWLout + $atribInsts.outputOWL;
                            }
                       }
                    $triplo.tabConcsout = $triplo.tabConcsin;
                  }
           | '('? c1=TEXT '=' ('pof'|'parte') '=>' c2=TEXT ')'?
                  { if (!(($triplo.concts.contains($c1.text))||($triplo.indivs.contains($c1.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> Nao E Conceito Nem Individuo"); }
                    if (!(($triplo.concts.contains($c2.text))||($triplo.indivs.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O objeto <"+$c2.text+"> Nao E Conceito Nem Individuo");  }
                    if (($triplo.indivs.contains($c1.text))&&(!($triplo.indivs.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> e o objeto <"+$c2.text+"> de POF Nao Sao Ambos Individuos"); }
                    if (($triplo.concts.contains($c1.text))&&(!($triplo.concts.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> e o objeto <"+$c2.text+"> de POF Nao Sao Ambos Conceitos");  }
                    $triplo.trip = $c1.text + ",pof," + $c2.text;
                    if (!$triplo.erroTriplo){
                       $triplo.outputDOTout = $triplo.outputDOTin + "\t\"" + $c1.text + "\"->\"" + $c2.text + "\" [label=\"pof\"]\n";

                       if (($triplo.indivs.contains($c1.text))&&($triplo.indivs.contains($c2.text)))//POF Sao Ambos Individuos
                       { $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyAssertion>\n\t\t<ObjectProperty IRI=\"#" + "pof" + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c1.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyAssertion>\n";  }
                       if (($triplo.concts.contains($c1.text))&&($triplo.concts.contains($c2.text)))//POF Sao Ambos conceitos
                        { $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyDomain>\n\t\t<ObjectProperty IRI=\"#"+ "pof" + "\"/>\n\t\t<Class IRI=\"#"+ $c1.text + "\"/>\n\t</ObjectPropertyDomain>\n"
                                               + "\t<ObjectPropertyRange>\n\t\t<ObjectProperty IRI=\"#"+ "pof" + "\"/>\n\t\t<Class IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyRange>\n";}
                    }
                    $triplo.tabConcsout = $triplo.tabConcsin;
                  }
           | '('? c1=TEXT '=' ('is-a'|'subclasse') '=>' c2=TEXT ')'?
                  { if (!($triplo.concts.contains($c1.text)))
                       { $triplo.erroTriplo = true;
                        System.out.println("ERRO: O sujeito <"+$c1.text+"> de ISA Nao E um Conceito"); }
                    if (!($triplo.concts.contains($c2.text)))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O objeto <"+$c2.text+"> de ISA Nao E um Conceito");  }
                    $triplo.trip = $c1.text + ",isa," + $c2.text;
                    if (!$triplo.erroTriplo){
                        /* Inferencia de atributos Herdados em subClasses */
                        if ($triplo.tabConcsin.containsKey($c2.text))
                          { lstAtribs = $triplo.tabConcsin.get($c2.text);
                            if ($triplo.tabConcsin.containsKey($c1.text)) {
                                List<String> lstAux = new ArrayList<String>($triplo.tabConcsin.get($c1.text));
                                for(String a : lstAtribs) { if (lstAux.indexOf(a)==-1) lstAux.add(a); }
                             $triplo.tabConcsin.put($c1.text,lstAux); }
                            else { $triplo.tabConcsin.put($c1.text,lstAtribs); }
                          }
                       $triplo.outputDOTout = $triplo.outputDOTin + "\t\"" + $c1.text + "\"->\"" + $c2.text + "\" [label=\"isa\" style=dashed color=blue]\n";
                       $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<SubClassOf>\n\t\t<Class IRI=\"#"+ $c1.text + "\"/>\n\t\t<Class IRI=\"#"+ $c2.text + "\"/>\n\t</SubClassOf>\n";
                    }
                    $triplo.tabConcsout = $triplo.tabConcsin;
                  }
           | '('? c1=TEXT '=' ('synonym'|'sinonimo') '=>' c2=TEXT ')'?
                  { if (!(($triplo.concts.contains($c1.text))||($triplo.rels.contains($c1.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> Nao E Conceito Nem Relacao"); }
                    if (!(($triplo.concts.contains($c2.text))||($triplo.rels.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O objeto <"+$c2.text+"> Nao E Conceito Nem Relacao");  }
                    if (! $triplo.erroTriplo ){
                    if (($triplo.rels.contains($c1.text))&&(!($triplo.rels.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> e o objeto <"+$c2.text+"> de uma SINONIMIA Nao Sao Ambos Relacoes"); }
                    if (($triplo.concts.contains($c1.text))&&(!($triplo.concts.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> e o objeto <"+$c2.text+"> de uma SINONIMIA Nao Sao Ambos Conceitos");  }
                    }
                    $triplo.trip = $c1.text + ",syn," + $c2.text;
                    if (!$triplo.erroTriplo){
                       $triplo.outputDOTout = $triplo.outputDOTin + "\t\"" + $c1.text + "\"->\"" + $c2.text + "\" [label=\"SYN\" dir=\"both\" style=dashed color=green]\n";
///vou aqui
                    //   if (($triplo.indivs.contains($c1.text))&&($triplo.indivs.contains($c2.text)))//POF Sao Ambos Individuos
                    //   { $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyAssertion>\n\t\t<ObjectProperty IRI=\"#" + "pof" + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c1.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyAssertion>\n";  }
                    //   if (($triplo.concts.contains($c1.text))&&($triplo.concts.contains($c2.text)))//POF Sao Ambos conceitos
                    //    { $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyDomain>\n\t\t<ObjectProperty IRI=\"#"+ "pof" + "\"/>\n\t\t<Class IRI=\"#"+ $c1.text + "\"/>\n\t</ObjectPropertyDomain>\n"
                    //                           + "\t<ObjectPropertyRange>\n\t\t<ObjectProperty IRI=\"#"+ "pof" + "\"/>\n\t\t<Class IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyRange>\n";}
                    }
                    $triplo.tabConcsout = $triplo.tabConcsin;
                  }

           | '('? c1=TEXT '=' rel=TEXT '=>' c2=TEXT ')'?
                  { if (!(($triplo.concts.contains($c1.text))||($triplo.indivs.contains($c1.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O sujeito <"+$c1.text+"> Nao E Conceito Nem Individuo"); }
                    if (!(($triplo.concts.contains($c2.text))||($triplo.indivs.contains($c2.text))))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O objeto <"+$c2.text+"> Nao E Conceito Nem Individuo");  }
                    if (!($triplo.rels.contains($rel.text)))
                       { $triplo.erroTriplo = true;
                         System.out.println("ERRO: O predicado <"+$rel.text+"> Nao E uma Relacao"); }
                    $triplo.trip = $c1.text + "," + $rel.text + "," + $c2.text;
                    if (!$triplo.erroTriplo){
                       $triplo.outputDOTout = $triplo.outputDOTin + "\t\"" + $c1.text + "\"->\"" + $c2.text + "\" [label=\"" + $rel.text + "\"]\n";

                       if (($triplo.indivs.contains($c1.text))&&($triplo.indivs.contains($c2.text)))//Sao Ambos Individuos
                          { $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyAssertion>\n\t\t<ObjectProperty IRI=\"#" + $rel.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c1.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyAssertion>\n";  }
                       else{
                           $triplo.outputOWLout = $triplo.outputOWLin + "\n\t<ObjectPropertyDomain>\n\t\t<ObjectProperty IRI=\"#"+ $rel.text + "\"/>\n\t\t<Class IRI=\"#"+ $c1.text + "\"/>\n\t</ObjectPropertyDomain>\n"
                                               + "\t<ObjectPropertyRange>\n\t\t<ObjectProperty IRI=\"#"+ $rel.text + "\"/>\n\t\t<Class IRI=\"#"+ $c2.text + "\"/>\n\t</ObjectPropertyRange>\n";}
                    }
                    $triplo.tabConcsout = $triplo.tabConcsin;
                  }
           | c1=TEXT '=[' relacionamentos ']'
           ;

relacionamentos : relacionamento=TEXT '=>' c2=TEXT (',' relacionamentos)* ','?
                | ('synonym'|'sinonimo') '=>' c2=TEXT (',' relacionamentos)* ','?
                | ('is-a'|'subclasse') '=>' c2=TEXT (',' relacionamentos)* ','?
                | ('pof'|'parte') '=>' c2=TEXT (',' relacionamentos)* ','?
                | ('iof'|'instancia') '=>' c2=TEXT atribInsts[$c2.text]? (',' relacionamentos)* ','?
                ;

atribInsts [String concNome]
           returns [String atribsI, int conta, HashSet<String> atrs,String outputDOT,String outputOWL]
@init { $atribInsts.atrs = new HashSet<String>(); }
           : '[' atribInst[$atribInsts.concNome]
                               { $atribInsts.atribsI = $atribInst.atributoInst;
                                 $atribInsts.conta   = 1;
                                 $atribInsts.atrs.add($atribInst.atr);
                                 $atribInsts.outputOWL = $atribInst.outputOWL; }
             (',' atribInst[$atribInsts.concNome]
                               { $atribInsts.atribsI = $atribInsts.atribsI + " ; " + $atribInst.atributoInst;
                                 $atribInsts.conta  += 1;
                                 $atribInsts.atrs.add($atribInst.atr);
                                 $atribInsts.outputOWL = $atribInsts.outputOWL + $atribInst.outputOWL; } )*
             ']'  {//$atribInsts.outputDOT =  "\"" +$atribInsts.atribsI+ "\" [shape=rectangle,color=goldenrod]; \n";
             }
           ;
atribInst [String concNome]
           returns [String atributoInst, String atr, String outputOWL]
           : atrib=TEXT '=' valor=TEXT
             {//$atribInst.atributoInst = $atrib.text + "=" + $valor.text;               $atribInst.atr = $atrib.text;              $atribInst.outputOWL = "\n\t<DataPropertyAssertion>\n\t\t<DataProperty IRI=\"#"+ $atrib.text + "\"/>\n\t\t<NamedIndividual IRI=\"#"+ $atribInst.concNome + "\"/>\n\t\t<Literal datatypeIRI=\"&xsd;string\">"+ $valor.text +"</Literal>\n\t</DataPropertyAssertion>\n" ;
             }
           ;

/* Definicao do Analisador LEXICO */
TEXT: ([a-zA-Z��������������������������������][-_/a-zA-Z0-9��������������������������������]*) | (('\''|'\"'|'(') ~('\''|'\"'|')')* ('\''|'\"'|')'));

NUM: '-'?[0-9]+;

Sep : [ \t\r\n]+ -> skip;

COMENT: '%'~('\r'|'\n')*[\r\n] -> skip;
