grammar OntoDLPlus;

@header{
        import java.util.*;
        import java.io.BufferedWriter;
        import java.nio.file.*;
        import java.nio.file.StandardOpenOption;
        import java.nio.file.attribute.FileAttribute;
        import java.nio.file.attribute.PosixFilePermission;
        import java.nio.file.attribute.PosixFilePermissions;
        import java.nio.charset.Charset;
        import java.nio.charset.StandardCharsets;
        import OntoDLPlus.*;
}

ontology returns[Ontology onto]
@init{Ontology rec = new Ontology();
      HashSet<DataProperty> atribList = new HashSet<DataProperty>();}
             : ('Ontologia'|'Ontology') TEXT {rec.setName($TEXT.text);}
                    atribs[rec, atribList]?
                    concepts[rec, atribList]
                    inviduals[rec]?
                    relations[rec]
                    triples[rec]
                    restrictions[rec]?
                 '.'  {$onto=rec;
                       $onto.normRelations();
                       String owl= $onto.odlp2OWL();
                       if(owl.length()!=0) {
                          String dot= $onto.odlp2DOT();
                          String pl= $onto.odlp2Prolog();
                          String als= $onto.odlp2Alloy();
                          Path current = Paths.get("");
                          String workDir = current.toAbsolutePath().toString();
                          String DOTpath = workDir + "/" + $onto.getName() + ".dot";
                          String OWLpath = workDir + "/" + $onto.getName() + ".owl";
                          String PLpath = workDir + "/" + $onto.getName() + ".pl";
                          String ALSpath = workDir + "/" + $onto.getName() + ".als";
                          Set<PosixFilePermission> perms = PosixFilePermissions.fromString("rwxrwxrwx");
                          FileAttribute<Set<PosixFilePermission>> attr = PosixFilePermissions.asFileAttribute(perms);
                          //System.out.println(Charset.defaultCharset());
                          System.setProperty("file.encoding","UTF-8");
                          //System.out.println(Charset.defaultCharset());
                          try {
                            if (!Files.exists(Paths.get(DOTpath))) {
                                Files.createFile(Paths.get(DOTpath));
                            } else {
                                Files.delete(Paths.get(DOTpath));
                                Files.createFile(Paths.get(DOTpath));
                            }BufferedWriter dotwriter = Files.newBufferedWriter(Paths.get(DOTpath), StandardCharsets.UTF_8);
                            dotwriter.append(dot);
                            dotwriter.flush();
                            //Files.write(Paths.get(DOTpath), dot.getBytes(), StandardOpenOption.CREATE, StandardCharsets.UTF_8);
                          } catch(Exception e) { e.printStackTrace(); }
                          PosixFilePermissions.asFileAttribute(perms);
                          try {
                            if (!Files.exists(Paths.get(OWLpath))) {
                                Files.createFile(Paths.get(OWLpath));
                            } else {
                                Files.delete(Paths.get(OWLpath));
                                Files.createFile(Paths.get(OWLpath));
                            }
                            BufferedWriter owlwriter = Files.newBufferedWriter(Paths.get(OWLpath), StandardCharsets.UTF_8);
                            owlwriter.append(owl);
                            owlwriter.flush();
                            //Files.write(Paths.get(OWLpath), owl.getBytes(), StandardOpenOption.CREATE, StandardCharsets.UTF_8);
                          } catch(Exception e) { e.printStackTrace(); }
                          try {
                            if (!Files.exists(Paths.get(PLpath))) {
                                Files.createFile(Paths.get(PLpath));
                            } else {
                                Files.delete(Paths.get(PLpath));
                                Files.createFile(Paths.get(PLpath));
                            }
                            BufferedWriter plwriter = Files.newBufferedWriter(Paths.get(PLpath), StandardCharsets.UTF_8);
                            plwriter.append(pl);
                            plwriter.flush();
                            //Files.write(Paths.get(PLpath), pl.getBytes(), StandardOpenOption.CREATE, StandardCharsets.UTF_8);
                          } catch(Exception e) { e.printStackTrace(); }
                          try {
                            if (!Files.exists(Paths.get(ALSpath))) {
                                Files.createFile(Paths.get(ALSpath));
                            } else {
                                Files.delete(Paths.get(ALSpath));
                                Files.createFile(Paths.get(ALSpath));
                            }
                            BufferedWriter alswriter = Files.newBufferedWriter(Paths.get(ALSpath), StandardCharsets.UTF_8);
                            alswriter.append(als);
                            alswriter.flush();
                            //Files.write(Paths.get(PLpath), als.getBytes(), StandardOpenOption.CREATE, StandardCharsets.UTF_8);
                          } catch(Exception e) { e.printStackTrace(); }

                      } else {
                            System.out.println("Semantic errors detected with specification. Discard any result and correct specification.");
                      }}
           ;


atribs[Ontology onto, HashSet<DataProperty> atribList]
           : dataProperties
             '{' a1=attribute {$atribList.add($a1.data.clone());}
             (',' a2=attribute{$atribList.add($a2.data.clone());})*
             '}'
           ;


concepts[Ontology onto, HashSet<DataProperty> atribList]
           : conceptsKey
             '{' c1=concept[$onto,$atribList] {$onto.addConcept($c1.conc);}
             (',' c2=concept[$onto,$atribList]{$onto.addConcept($c2.conc);})*
             '}'
           ;


concept [Ontology onto, HashSet<DataProperty> atribList] returns[Concept conc]
           : TEXT { $conc=new Concept($TEXT.text); }
             attributes[$onto,$conc,$atribList]?
           ;

attributes  [Ontology onto, Concept conc, HashSet<DataProperty> atribList]
@init {  }
           : '[' (a1=attribute {$conc.addDataProperty($a1.data.clone());} | sa1=simpAtrib[$atribList]{
                        if($sa1.data!=null) $conc.addDataProperty($sa1.data.clone());
                        else $onto.addSemError("Declaração de atributo <"+$sa1.txt+"inválida, tipo de atributo não declarado.");
           } )
             (','(a2=attribute {$conc.addDataProperty($a2.data.clone());} | sa2=simpAtrib[$atribList]{
                        if($sa2.data!=null) $conc.addDataProperty($sa2.data.clone());
                        else $onto.addSemError("Declaração de atributo <"+$sa2.txt+"inválida, tipo de atributo não declarado.");
           }))*
             ']'
           ;

simpAtrib [HashSet<DataProperty> atribList] returns [String txt, DataProperty data]
           : atr=TEXT {$txt=$atr.text;
                       $data=null;
                       for(DataProperty prop: $atribList) {
                            if(prop.getPropName().equals($atr.text)) $data=prop.clone();
                       }}
           ;

attribute returns [DataProperty data]
           : atr=TEXT ':' tipo {$data=new DataProperty($atr.text, $tipo.txt, $tipo.restricted, $tipo.opts, true);}
           ;

tipo returns [String txt, boolean restricted, List<String> opts] 
@init{ $opts = new ArrayList(); }
     : 'string' {$restricted=false; $txt="xsd:string";}
     | 'int'    {$restricted=false; $txt="xsd:int";/* confirmar */} 
     | 'enum' '(' e1=(TEXT|NUM) {$opts.add($e1.text); $restricted=true; $txt="list";}
       (',' e2=(TEXT|NUM)       {$opts.add($e2.text); $restricted=true; $txt="list";})*
       ')' 
;

inviduals[Ontology onto]
           : ('individuos'|'individuals')
             '{' TEXT  {$onto.addIndividual($TEXT.text);}
             (',' TEXT {$onto.addIndividual($TEXT.text);})*
             '}'
           ;

relations[Ontology onto]
    @init{$onto.addRelation("isa");
          $onto.addRelation("pof");
          $onto.addRelation("synonym");
          $onto.addRelation("iof");
          Relation r;
         }
           :objectProps
             '{' r1=TEXT {r=new Relation($r1.text);}  ('[' relInf[$onto,r] (',' relInf[$onto,r])* ']' )? {if(r.getDomain()==null & r.getCodomain()==null & r.getProps().size()==0) {$onto.addRelation($r1.text);}
                                                                                                          else $onto.addRelation($r1.text, r.getDomain(), r.getCodomain(), r.getProps());}
             (',' r2=TEXT {r=new Relation($r2.text);}  ('[' relInf[$onto,r] (',' relInf[$onto,r])* ']' )? {if(r.getDomain()==null & r.getCodomain()==null & r.getProps().size()==0) {$onto.addRelation($r2.text);}
                                                                                                           else $onto.addRelation($r2.text, r.getDomain(), r.getCodomain(), r.getProps());} )*
             '}'
           ;



relInf[Ontology onto, Relation r]
@init{Set<String> props= new HashSet<String>();}
      : domain ':' d1=TEXT             {r.setDomain($d1.text);}
      | counterdomain ':' c1=TEXT       {r.setCodomain($c1.text);}
      | 'props' ':' properties[$onto,props] {r.addProps($properties.props);}
      ;


properties [Ontology onto, Set<String> res] returns [Set<String> props]
            : '(' p1=property[$res] {$res=$p1.prop;} (',' p2=property[$res]{$res=$p2.prop;})* ')' {$props=$res;}
            | p1=property[$res] {$props=$p1.prop;}
            ;

property [Set<String> res] returns [Set<String> prop]
            : (('bijecao') | ('bijection'))       {$res.add("injectiva");
                               $res.add("inteira");
                               $res.add("simples");
                               $res.add("sobrejectiva");
                               $prop=$res;}
            | (('injecao') | ('injection'))       {$res.add("injectiva");
                               $res.add("inteira");
                               $res.add("simples");
                               $prop=res;}
            | (('sobrejecao') | ('surjection'))      {$res.add("inteira");
                               $res.add("simples");
                               $res.add("sobrejectiva");
                               $prop=res;}
            | (('abstracao') | ('abstraction'))     {$res.add("simples");
                               $res.add("sobrejectiva");
                               $prop=res;}
            | (('funcao') | ('function'))        {$res.add("inteira");
                               $res.add("simples");
                               $prop=res;}
            | (('representacao') | ('representation'))  {$res.add("injectiva");
                               $res.add("inteira");
                               $prop=res;}
            | (('sobrejetiva') | ('surjective'))      {$res.add("sobrejectiva"); $prop=$res;}
            | (('simples') | ('simple'))          {$res.add("simples"); $prop=$res;}
            | (('inteira') | ('entire'))          {$res.add("inteira"); $prop=$res;}
            | (('injetiva') | ('injective'))       {$res.add("injectiva"); $prop=$res;}
            | (('inversaDe') | ('inverseOf')) '=' rel=TEXT {$res.add("inversa="+$rel.text); $prop=$res;}
            ;

triples   [Ontology onto]
          : ('triplos'|'triples')
             '{' triple[$onto]
             (';' triple[$onto] )* '}'
           ;

triple     [Ontology onto]
           : '('? c1=TEXT '=' iof '=>' c2=TEXT {$onto.createTriple($c1.text,"iof",$c2.text);} atribInsts[$onto, $c1.text, $c2.text]? ')'?
           | '('? c1=TEXT '=' pof '=>' c2=TEXT {$onto.createTriple($c1.text,"pof",$c2.text);}')'?
           | '('? c1=TEXT '=' isa '=>' c2=TEXT {$onto.createTriple($c1.text,"isa",$c2.text);} ')'?
           | '('? c1=TEXT '=' syn '=>' c2=TEXT {$onto.createTriple($c1.text,"synonym",$c2.text);}')'?
           | '('? c1=TEXT '=' rel=TEXT relAtrib? '=>' c2=TEXT {$onto.createTriple($c1.text,$rel.text,$c2.text);}')'?
           | c1=TEXT '=[' relationship[$onto, $c1.text] (';' relationship[$onto,$c1.text])* ']'
           ;

relationship [Ontology onto, String leftSide]
                : rel=TEXT relAtrib? '=>' c2=TEXT {$onto.createTriple($leftSide, $rel.text, $c2.text);} (',' multiRel[$onto, $leftSide, $rel.text])*
                | syn '=>' c2=TEXT {$onto.createTriple($leftSide, "synonym", $c2.text);} (',' multiRel[$onto, $leftSide, "synonym"])*
                | isa '=>' c2=TEXT {$onto.createTriple($leftSide, "isa", $c2.text);} (',' multiRel[$onto, $leftSide, "isa"])*
                | pof '=>' c2=TEXT {$onto.createTriple($leftSide, "pof", $c2.text);} (',' multiRel[$onto, $leftSide, "pof"])*
                | iof '=>' c2=TEXT {$onto.createTriple($leftSide, "iof", $c2.text);} atribInsts[$onto, $leftSide, $c2.text]? (',' multiRel[$onto, $leftSide, "iof"])*
                ;

relAtrib : '[' TEXT '=' (TEXT|NUM) ']';

multiRel [Ontology onto, String leftSide, String relation]
                       : c2=TEXT {$onto.createTriple($leftSide, $relation, $c2.text);};

atribInsts [Ontology onto, String individuo, String conc]
@init {int expected= $onto.getConceptNAtr($conc);
       int found= 0;}
           : '[' atribInst[$onto, $individuo, $conc]  {found++;}
             (',' atribInst[$onto, $individuo, $conc] {found++;})*
             ']'
             {if(found!=expected) {
               $onto.addSemError("ERROR: The number of atributes of instance "+ $individuo + "(" + found +") doesn't match the expected number of atributes of concept" + $conc + " (" + expected + ").");
             }}
           ;
atribInst [Ontology onto, String individuo, String conc]
           : atrib=TEXT '=' valor=(TEXT|NUM) {if($onto.conceptContainsAtrib($conc, $atrib.text)) {
                                                $onto.genDataProperty($atrib.text,$individuo,$conc,$valor.text);
                                              } else {
                                                $onto.addSemError("ERROR: Atribute <"+$atrib.text+"> of instance <"+$individuo+"> is not part of atributes of its' respective concept <"+$conc+">.");
                                              }}
            ;

restrictions [Ontology onto] 
           : ('restricoes'|'restrictions') '{' r1=restriction[$onto] ':-' g1=guards[$onto] '.' {$onto.addRestriction($r1.left,$r1.relation,$r1.right,$g1.res);}
             (r2=restriction[$onto] ':-' g2=guards[$onto] '.' {$onto.addRestriction($r2.left,$r2.relation,$r2.right,$g2.res);})*
           '}'
           ;

restriction [Ontology onto]
  returns [String left, String relation, String right]: 
          conc=TEXT '=' rel=TEXT '=>' rgt=TEXT {$left=$conc.text;$relation=$rel.text;$right=$rgt.text;}
          ; 

guards [Ontology onto]
returns [Restriction res]
        :
        d=disjunctions[$onto] {$res=$d.res.clone();}
        ;

disjunctions [Ontology onto]
    returns[Restriction res]
           : c1=conjunctions[$onto] {$res=$c1.res.clone();} (';' c2=conjunctions[$onto] {$res.addDisjunction($c2.res);})*
           ;

conjunctions [Ontology onto]
    returns[Restriction res]
@init{boolean neg=false;}       
           : (not {neg=!neg;})* '(' d1=disjunctions[$onto] {if(neg) {$res=$onto.createRestriction(null,true);
                                                                   $res.addConjunction($d1.res.clone());}
                                                          else $res=$d1.res;} ')' 
             (c=conjunction[$onto] {$c.res.addConjunction($res); $res=$c.res.clone();})*
           | g1=guard[$onto] {$res=$g1.res.clone();} (c=conjunction[$onto] {$res.addConjunction($c.res.clone());})*
           ;

conjunction [Ontology onto]
   returns[Restriction res]
@init{boolean neg=false;}      
          : ',' g1=guard[$onto] {$res=$g1.res.clone();}
          | ',' (not {neg=!neg;})* '(' d1=disjunctions[$onto] ')' { if(neg) {
                                                                            $res=$onto.createRestriction(null, !neg);
                                                                            $res.addConjunction($d1.res.clone());
                                                                          } 
                                                                  else {$res=$d1.res.clone();}
                                                                }
          ;

guard [Ontology onto]
       returns [Restriction res] 
       @init{boolean igualdade=false;}
        : not  g1=guard[$onto]                               {$res=$g1.res.clone(); $res.changeNegation();}
        | left=TEXT '=' rel=TEXT '=>' right=TEXT              {$res= $onto.createRestriction($left.text,$rel.text,$right.text);}
        | left=TEXT '=' syn '=>' right=TEXT              {$res= $onto.createRestriction($left.text,"synonym",$right.text);}
        | left=TEXT '=' isa '=>' right=TEXT              {$res= $onto.createRestriction($left.text,"isa",$right.text);}
        | left=TEXT '=' pof '=>' right=TEXT              {$res= $onto.createRestriction($left.text,"pof",$right.text);}
        | left=TEXT '=' iof '=>' right=TEXT              {$res= $onto.createRestriction($left.text,"iof",$right.text);}
        | left=TEXT ('=='{igualdade=true;}|'\\=' ) right=TEXT {if (igualdade) $res= $onto.createRestriction($left.text,"==",$right.text);
                                                               else $res= $onto.createRestriction($left.text,"\\=",$right.text);}
        ;


domain: 'domain'
       |'dominio';

counterdomain: 'contradominio'
             |'codiminio'
             |'counterdomain'
             |'codomain'
             ;

conceptsKey: 'conceitos'
           |'concepts'
           |'classes'
           ;

objectProps: 'relacoes'
            |'relationships'
            |'objectProperties'
            |'propriedadesObjetos'
            ;

dataProperties: 'atributos'
         |'attributes'
         |'dataProperties'
         |'propriedadesDados'
         ;

not: 'not'
    |'!'
    |'nao';

isa: 'is-a'
    |'subclasse'
    |'a'
    |'subClassOf';

iof: 'iof'
    |'instancia'
    |'instanceof'
    |'type';

syn: 'synonym'
    |'sinonimo';

pof: 'pof'
    |'parte'
    |'partOf';

/* Definicao do Anal('is-a'|'subclasse'|'a'|'subClassOf')dor LEXICO */
//TEXT: ([a-zA-ZÀ-ÏÑ-ÖÙ-Ýà-ïñ-öù-ÿ][-_/a-zA-Z0-9À-ÏÑ-ÖÙ-Ýà-ïñ-öù-ÿ]*) | (('\"'|'\''|'(') ~('\"'|'\''|')')* ('\"'|'\''|')') );

TEXT: (('a'..'z'|'A'..'Z'|'À'..'Ï'|'Ñ'..'Ö'|'Ù'..'Ý'|'à'..'ï'|'ñ'..'ö'|'ù'..'ÿ')('-'|'_'|'/'|'a'..'z'|'A'..'Z'|'0'..'9'|'À'..'Ï'|'Ñ'..'Ö'|'Ù'..'Ý'|'à'..'ï'|'ñ'..'ö'|'ù'..'ÿ')*) | (('"'|'\'') ~('"'|'\'')* ('"'|'\'') );


NUM: '-'?('0'..'9')+;

Sep : (' '|'\t'|'\r'|'\n')+ -> skip;// {skip();};

COMENT: '%'~('\r'|'\n')*('\r'|'\n')  -> skip;


