package OntoDLPlus;

import OntoDLPlus.*;
import java.util.HashSet;
import java.util.Iterator;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.lang.StringBuilder;

public class Concept {
    private final String concept;
    private List<DataProperty> tabelaAtributos;

    public Concept(final String c) {
        this.concept = c;
        this.tabelaAtributos = new ArrayList<DataProperty>();
    }

    public String getConcept() {return this.concept;}
    public int getAtrLength() {return this.tabelaAtributos.size();}
    public List<DataProperty> getProperties () {return this.tabelaAtributos;}

    public void addDataProperty(DataProperty a) {
        this.tabelaAtributos.add(a.clone());
    }

    public DataProperty getProperty (String s) {
        for(DataProperty d: this.tabelaAtributos) {
            if(d.getPropName().equals(s)) {
                return d.clone();
            }
        }
        return null;
    }

    public boolean containsAtrib(String s) {
        for(DataProperty d: this.tabelaAtributos) {
            if(d.getPropName().equals(s)) return true;
        }
        return false;
    }

    public String odlp2OWL() {
        StringBuilder s= new StringBuilder();
        s.append("\n\t<Declaration>");
        s.append("\n\t\t<Class IRI=\"#");
        s.append(this.concept);
        s.append("\"/>");
        s.append("\n\t</Declaration>");
        return s.toString();
    }

    public String atribs2DOT () {
        StringBuilder s= new StringBuilder();
        Iterator i = this.tabelaAtributos.iterator();
        DataProperty d = (DataProperty) i.next();
        s.append(d.getPropName());
        while(i.hasNext()) {
            d= (DataProperty) i.next();
            s.append(";");
            s.append(d.getPropName());
        }
        return s.toString();
    }

    public String odlp2Dot() {
        StringBuilder s= new StringBuilder();
        s.append("\"");
        s.append(this.concept);
        s.append("\" [shape = ellipse, style=filled, color=turquoise4]; \n");
        if (this.tabelaAtributos.size()>0) {
            s.append("\"");
            s.append(atribs2DOT());
            s.append("\" [shape = rectangle, color=turquoise4]; \n");
            s.append("\t\"");
            s.append(this.concept);
            s.append("\"->\"");
            s.append(atribs2DOT());
            s.append("\" [label=\"Properties\" style=dotted color=red]\n");
        }
        return s.toString();
    }
}
