package OntoDLPlus;

import OntoDLPlus.*;
import java.util.HashSet;
import java.util.HashMap;
import java.util.List;
import java.lang.StringBuilder;
import java.util.ArrayList;

public class DataProperty implements Comparable<DataProperty>{
    private final String propName;
    private String indName;
    private String type;
    private String value;
    private boolean restricted;
    private List<String> restrictions;
    private final boolean forcedProp;

    public DataProperty(final String p, final String i, final String t, boolean r, List<String> rlist, final String v, final boolean f) {
        this.propName = p;
        this.indName = i;
        this.type = t;
        this.restricted = r;
        this.restrictions = rlist;
        this.value = v;
        this.forcedProp = f;
    }

    public DataProperty(final String p, final String t, boolean r, List<String> rlist, final boolean f) {
        this.propName = p;
        this.indName = null;
        this.type = t;
        this.restricted = r;
        this.restrictions = rlist;
        this.value = null;
        this.forcedProp = f;
    }

    public DataProperty(final String p, final String i, final String t, boolean r, List<String> rlist, final boolean f) {
        this.propName = p;
        this.indName = i;
        this.type = t;
        this.restricted = r;
        this.restrictions = rlist;
        this.value = null;
        this.forcedProp = f;
    }

    public DataProperty clone() {
        return new DataProperty(this.propName, this.indName, this.type, this.restricted, this.restrictions, this.value, this.forcedProp);
    }

    public void setValue(final String v) throws Exception {
        if(!this.restricted) {
            this.value = v;
        } else {
            if(this.restrictions.contains(v)) {
                this.value= v;
            }
            else {
                throw new Exception("Invalid value for property "+this.propName+".");
            }
        }
    }

    public void setIndName(final String i) {
        this.indName = i;
    }

    public void setType(String s) {
        this.type=s;
    }

    public boolean equals(final Object o) {
        if (o == this)
            return true;
        if (o == null || o.getClass() != this.getClass())
            return false;
        final DataProperty d = (DataProperty) o;
        if (!this.propName.equals(d.getPropName()))
            return false;
        if (!this.indName.equals(d.getIndName()))
            return false;
        if (!this.value.equals(d.getValue()))
            return false;
        if (!this.restricted != d.getRestricted())
            return false;
        if (this.forcedProp != d.getForceProp())
            return false;
        if (!this.type.equals(d.getType()))
            return false;
        return true;
    }

    public String getPropName() {
        return this.propName;
    }

    public String getType() {
        return this.type;
    }

    public boolean getRestricted() {
        return this.restricted;
    }

    public List<String> getRestrictions () {
        return this.restrictions; 
    }

    public String getIndName() {
        return this.indName;
    }

    public String getValue() {
        return this.value;
    }

    public boolean getForceProp() {
        return this.forcedProp;
    }

    public String odlp2OWL() {
        final StringBuilder s = new StringBuilder();
        s.append("\n\t<DataPropertyAssertion>\n\t\t<DataProperty IRI=\"#");
        s.append(this.propName);
        s.append("\"/>\n\t\t<NamedIndividual IRI=\"#");
        s.append(this.indName);
        s.append("\"/>\n\t\t<Literal datatypeIRI=\"");
        s.append(this.getType());
        s.append("\">" + this.getValue() + "</Literal>\n\t</DataPropertyAssertion>\n");
        return s.toString();
    }

    public String odlp2Dot() {
        final StringBuilder s = new StringBuilder();
        s.append("\n\"");
        s.append(this.propName);
        s.append("=");
        s.append(this.value.replaceAll("\"",""));
        s.append("\" [shape=rectangle,color=goldenrod]; \n");
        s.append("\t\"");
        s.append(this.indName);
        s.append("\"->\"");
        s.append(this.propName);
        s.append("=");
        s.append(this.value.replaceAll("\"",""));
        s.append("\" [label=\"properties\" style=dotted color=red]\n");
        return s.toString();
    }

    @Override
    public int compareTo(DataProperty o) {
        if(this.indName==null || o.getIndName()==null) {
            if(this.indName!=null) {return 1;}
            if (o.getIndName()!=null) {return -1;}
            return this.propName.compareTo(o.getPropName());
        }
        if(this.indName.equals(o.getIndName())) {
            return this.propName.compareTo(o.getPropName());
        }
        return this.indName.compareTo(o.getIndName());
    }
}
