package OntoDLPlus;

import java.util.ArrayList;
import java.util.List;

public class Restriction {
    
    private Restriction conjunctions;
    private Restriction disjunctions;
    private Triple atomicRule;
    private boolean negative;

    public Restriction (Triple t, boolean n) {
        if(t!=null) this.atomicRule = t.clone();
        else this.atomicRule=null;
        this.negative = n;
        this.conjunctions = null;
        this.disjunctions = null;
    }

    public Restriction getConjunctions() {if(conjunctions!=null) return conjunctions.clone();
                                          else return null;}
    public Restriction getDisjunctions() {if(disjunctions!=null) return disjunctions.clone();
                                          else return null;}
    public Triple getTriple () {if(atomicRule!=null) return atomicRule.clone();
                                else return null;}
    public boolean getNegative () {return this.negative;}

    public void addConjunction (Restriction r) {
        if(this.conjunctions==null) {
            this.conjunctions = r.clone();
        } else {
            this.conjunctions.addConjunction(r.clone());
        }
    }

    public void addDisjunction (Restriction r) {
        if(this.disjunctions==null) {
            this.disjunctions = r.clone();
        } else {
            this.disjunctions.addDisjunction(r.clone());
        }
    }

    public void changeNegation () {
        this.negative = !this.negative;
    }


    @Override
    public String toString () {
        StringBuilder s = new StringBuilder();
        s.append("(");
        if(this.negative) 
            s.append("(not");
        if(this.atomicRule!=null) {
            s.append(this.atomicRule.toString());
        }
        if(this.negative) 
            s.append(")");
        if(this.conjunctions!=null) {
            s.append("AND");
            s.append(this.conjunctions.toString());
        }
        s.append(")");
        if(this.disjunctions!=null) {
            s.append("OR");
            s.append(this.disjunctions);
        }
        return s.toString();
    }

    public Restriction clone() {
        Restriction res = new Restriction(this.atomicRule, this.negative);
        if(this.conjunctions!=null) res.addConjunction(this.conjunctions.clone());
        if(this.disjunctions!=null) res.addDisjunction(this.disjunctions.clone());
        return res;
    }

    public String atom2Prolog(String atom, Ontology onto) {
        StringBuilder s = new StringBuilder();
        if(onto.containsConcept(atom)) {
            s.append("conceito(");
            s.append(atom);
            s.append(")");
        } else if(onto.containsIndividual(atom)) {
            s.append("individuo(");
            s.append(atom);
            s.append(")");
        } else {
            s.append(atom);
        }
        return s.toString();
    }

    public String odlp2Prolog (Ontology onto) {
        StringBuilder s = new StringBuilder();
        if(this.negative) {
            s.append("not(");
            if(this.atomicRule!=null) {
                if(!this.atomicRule.getRelation().equals("==") && !this.atomicRule.getRelation().equals("\\=")) {
                    s.append(this.atomicRule.getRelation());
                    s.append("(");
                    s.append(atom2Prolog(this.atomicRule.getLeft(), onto));
                    s.append(",");
                    s.append(atom2Prolog(this.atomicRule.getRight(), onto));
                    s.append("))");
                } else {
                    s.append(atom2Prolog(this.atomicRule.getLeft(), onto));
                    s.append(this.atomicRule.getRelation());
                    s.append(atom2Prolog(this.atomicRule.getRight(), onto));
                }
                if (this.conjunctions!=null) {
                    s.append(",");
                    s.append(this.conjunctions.odlp2Prolog(onto));
                }
                if (this.disjunctions!=null) {
                    s.append(";");
                    s.append(this.disjunctions.odlp2Prolog(onto));
                }
            } else {
                s.append(this.conjunctions.odlp2Prolog(onto));
                s.append(")");
                if(this.disjunctions!=null) {
                    s.append(";\n");
                    s.append(this.disjunctions.odlp2Prolog(onto));
                }
            }
        } else {
            if(this.atomicRule!=null) {
                if(!this.atomicRule.getRelation().equals("==") && !this.atomicRule.getRelation().equals("\\=")) {
                    s.append(this.atomicRule.getRelation());
                    s.append("(");
                    s.append(atom2Prolog(this.atomicRule.getLeft(),onto));
                    s.append(", ");
                    s.append(atom2Prolog(this.atomicRule.getRight(),onto));
                    s.append(")");
                } else {
                    s.append(atom2Prolog(this.atomicRule.getLeft(),onto));
                    s.append(this.atomicRule.getRelation());
                    s.append(atom2Prolog(this.atomicRule.getRight(),onto));
                }
                if (this.conjunctions!=null) {
                    s.append(", (");
                    s.append(this.conjunctions.odlp2Prolog(onto));
                    s.append(")");
                }
                if (this.disjunctions!=null) {
                    s.append("; ");
                    s.append(this.disjunctions.odlp2Prolog(onto));
                }
            } else {
                s.append(this.conjunctions.odlp2Prolog(onto));
                if(this.disjunctions!=null) {
                    s.append(";\n");
                    s.append(this.disjunctions.odlp2Prolog(onto));
                }
            }
        }
        return s.toString();
    }
}
