package OntoDLPlus;

import OntoDLPlus.*;
import java.util.HashSet;
import java.util.HashMap;
import java.util.List;
import java.lang.StringBuilder;

public class Triple implements Comparable<Triple>{
    private final String left;
    private final String right;
    private final String relation;
    private String ltype;
    private String rtype;

    public Triple(final String l, final String rel, final String rgt, final String lt, final String rt) {
        this.left = l;
        this.right = rgt;
        this.relation = rel;
        this.ltype = lt;
        this.rtype = rt;
    }

    public Triple(final String l, final String rel, final String rgt, Ontology o) {
        this.left = l;
        this.right = rgt;
        this.relation = rel;
        if (o.containsConcept(l)) {
            this.ltype = "concept";
        } else if (o.containsIndividual(l)) {
            this.ltype = "individuo";
        } else if (o.containsRelation(l)) {
            this.ltype = "relação";
        } else {this.ltype= "";}
        if (o.containsConcept(rgt)) {
            this.rtype = "concept";
        } else if (o.containsIndividual(rgt)) {
            this.rtype = "individuo";
        } else if (o.containsRelation(rgt)) {
            this.rtype = "relação";
        } else {this.rtype="";}
    }

    public Triple(final String l, final String rel, final String rgt) {
        this.left = l;
        this.right = rgt;
        this.relation = rel;
    }


    public Triple clone() {
        return new Triple(this.left, this.relation, this.right, this.ltype, this.rtype);
    }

    public boolean equals(final Object o) {
        if (o == this)
            return true;
        if (o == null || o.getClass() != this.getClass())
            return false;
        final Triple t = (Triple) o;
        if (!this.left.equals(t.getLeft()))
            return false;
        if (!this.right.equals(t.getRight()))
            return false;
        if (!this.relation.equals(t.getRelation()))
            return false;
        return true;
    }

    public String getLeft() {
        return this.left;
    }

    public String getRight() {
        return this.right;
    }

    public String getRelation() {
        return this.relation;
    }

    public String triple2OWL() {
        StringBuilder r = new StringBuilder();
        // isa relation
        if (this.relation.equals("isa")) {
            r.append("\n\t<SubClassOf>\n\t\t<Class IRI=\"#");
            r.append(this.left);
            r.append("\"/>\n\t\t<Class IRI=\"#");
            r.append(this.right);
            r.append("\"/>\n\t</SubClassOf>\n");
        }
        // iof relation
        else if (this.relation.equals("iof")) {
            r.append("\n\t<ClassAssertion>\n\t\t<Class IRI=\"#");
            r.append(this.right);
            r.append("\"/>\n");
            r.append("\t\t<NamedIndividual IRI=\"#");
            r.append(this.left);
            r.append("\"/>\n\t</ClassAssertion>\n");
        }
        // pof
        else if (this.relation.equals("pof")) {
            if (this.ltype.equals("individuo") && this.rtype.equals("individuo")) {
                // relação composta por 2 indivíduos
                r.append("\n\t<ObjectPropertyAssertion>\n\t\t");
                r.append("<ObjectProperty IRI=\"#pof\"/>\n\t\t");
                r.append("<NamedIndividual IRI=\"#");
                r.append(this.left);
                r.append("\"/>\n\t\t");
                r.append("<NamedIndividual IRI=\"#");
                r.append(this.right);
                r.append("\"/>\n\t");
                r.append("</ObjectPropertyAssertion>\n");
            }

            if (this.ltype.equals("concept") && this.rtype.equals("concept")) {
                // relação composta por 2 concepts
                r.append("\n\t<ObjectPropertyDomain>\n\t\t");
                r.append("<ObjectProperty IRI=\"#pof\"/>\n\t\t");
                r.append("<Class IRI=\"#");
                r.append(this.left);
                r.append("\"/>\n\t\t");
                r.append("</ObjectPropertyDomain>\n");
                r.append("\t<ObjectPropertyRange>\n\t\t");
                r.append("<ObjectProperty IRI=\"#pof\"/>\n\t\t");
                r.append("<Class IRI=\"#");
                r.append(this.right);
                r.append("\"/>\n\t");
                r.append("</ObjectPropertyRange>\n");
            }
        } else if (this.relation.equals("synonym")) {
            r.append("\n\t<EquivalentClasses>");
            r.append("\n\t\t<Class IRI=\"#");
            r.append(this.left);
            r.append("\"/>");
            r.append("\n\t\t<Class IRI=\"#");
            r.append(this.right);
            r.append("\"/>");
            r.append("\n\t</EquivalentClasses>\n");
        }

        else {
            if (this.ltype.equals("individuo") && this.rtype.equals("individuo")) {
                r.append("\n\t<ObjectPropertyAssertion>\n\t\t");
                r.append("<ObjectProperty IRI=\"#");
                r.append(this.relation);
                r.append("\"/>\n\t\t");
                r.append("<NamedIndividual IRI=\"#");
                r.append(this.left);
                r.append("\"/>\n\t\t");
                r.append("<NamedIndividual IRI=\"#");
                r.append(this.right);
                r.append("\"/>\n\t");
                r.append("</ObjectPropertyAssertion>\n");
            } else {
                r.append("\n\t<ObjectPropertyDomain>\n\t\t");
                r.append("<ObjectProperty IRI=\"#");
                r.append(this.relation);
                r.append("\"/>\n\t\t");
                r.append("<Class IRI=\"#");
                r.append(this.left);
                r.append("\"/>\n\t\t");
                r.append("</ObjectPropertyDomain>\n");
                r.append("\t<ObjectPropertyRange>\n\t\t");
                r.append("<ObjectProperty IRI=\"#");
                r.append(this.relation);
                r.append("\"/>\n\t\t");
                r.append("<Class IRI=\"#");
                r.append(this.right);
                r.append("\"/>\n\t");
                r.append("</ObjectPropertyRange>\n");
            }
        }

        return r.toString();
    }

    public String odlp2Dot() {
        final StringBuilder r = new StringBuilder();
        //isa relation
        if(this.relation.equals("isa")){
            r.append("\t\"");
            r.append(this.left);
            r.append("\" -> \"");
            r.append(this.right);
            r.append("\" [label=\"isa\" style=dashed color=blue];\n");
        }
        //iof relation
        else if(this.relation.equals("iof")){
            r.append("\t\"");
            r.append(this.left);
            r.append("\" -> \"");
            r.append(this.right);
            r.append("\" [label=\"iof\" style=dashed];\n");
        }
        //pof
        else if(this.relation.equals("pof")) {
            r.append("\t\"");
            r.append(this.left);
            r.append("\"->\"");
            r.append(this.right);
            r.append("\" [label=\"pof\"];\n");
        }
        else if(this.relation.equals("synonym")) {
            r.append("\t\"");
            r.append(this.left);
            r.append("\"->\"");
            r.append(this.right);
            r.append("\" [label=\"SYN\" dir=\"both\" style=dashed color=green];\n");
        }

        else {
            r.append("\t\"");
            r.append(this.left);
            r.append("\"->\"");
            r.append(this.right);
            r.append("\" [label=\"");
            r.append(this.relation);
            r.append("\"];\n");
        }

        return r.toString();
    }

    public String toString() {
        final StringBuilder r = new StringBuilder();
        r.append("Left: ");
        r.append(this.left);
        r.append(";\n");
        r.append("Relation: ");
        r.append(this.relation);
        r.append(";\n");
        r.append("Right: ");
        r.append(this.right);
        r.append(";\n");
        r.append("Left Type: ");
        r.append(this.ltype);
        r.append(";\n");
        r.append("Right Type: ");
        r.append(this.rtype);
        r.append(";\n");
        return r.toString();
    };

    @Override
    public int compareTo(Triple o) {
        if(!this.getRelation().equals(o.getRelation())) return this.getRelation().compareTo(o.getRelation());
        if(!this.getLeft().equals(o.getLeft())) return this.getLeft().compareTo(o.getLeft());
        if(!this.getRight().equals(o.getRight())) return this.getRight().compareTo(o.getRight());
        return 0;
    }
}
